EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'ServerRoles_CreateOrUpdateServerRole',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ 'ServerRole_Id',
    /* Description_Text        */ 'Creates or updates a server role.'
GO

IF OBJECT_ID('[ServerRoles_CreateOrUpdateServerRole]') IS NOT NULL DROP PROCEDURE [ServerRoles_CreateOrUpdateServerRole]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [ServerRoles_CreateOrUpdateServerRole]
(
	@ServerRole_Name NVARCHAR(50),
    @Description_Text NVARCHAR(MAX) = NULL,
	@Raft_Id INT = NULL,
    @ServerIds_Csv VARCHAR(MAX) = NULL,
    @DependsOnServerRoleIds_Csv VARCHAR(MAX) = NULL,
	@CollectOperations_Csv VARCHAR(MAX) = NULL,
    @ServerRole_Id INT = NULL OUT
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF (@ServerRole_Id IS NULL)
	BEGIN

		INSERT INTO [ServerRoles] ([ServerRole_Name], [Description_Text], [Raft_Id], [CollectOperations_Csv])
		VALUES (@ServerRole_Name, NULLIF(@Description_Text, ''), @Raft_Id, @CollectOperations_Csv)

		SET @ServerRole_Id = SCOPE_IDENTITY()

	END ELSE BEGIN

		UPDATE [ServerRoles]
		   SET [ServerRole_Name] = @ServerRole_Name,
               [Description_Text] = NULLIF(@Description_Text, ''),
		       [Raft_Id] = @Raft_Id,
			   [CollectOperations_Csv] = COALESCE(@CollectOperations_Csv, [CollectOperations_Csv])
		 WHERE [ServerRole_Id] = @ServerRole_Id

	END

    IF @ServerIds_Csv IS NOT NULL
    BEGIN

        DELETE [ServerRoleServers] 
         WHERE [ServerRole_Id] = @ServerRole_Id

        INSERT [ServerRoleServers] ([Server_Id], [ServerRole_Id])
        SELECT CAST([Value_Text] AS INT), @ServerRole_Id 
          FROM dbo.CsvToTable(@ServerIds_Csv, ',')

    END

    IF @DependsOnServerRoleIds_Csv IS NOT NULL
    BEGIN

        DELETE [ServerRoleDependencies] 
         WHERE [ServerRole_Id] = @ServerRole_Id

        INSERT [ServerRoleDependencies] ([ServerRole_Id], [DependsOn_ServerRole_Id])
        SELECT @ServerRole_Id, CAST([Value_Text] AS INT)
          FROM dbo.CsvToTable(@DependsOnServerRoleIds_Csv, ',')

    END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH


END
GO

GRANT EXECUTE ON [ServerRoles_CreateOrUpdateServerRole] TO [OtterUser_Role]
