EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Users_CreateOrUpdateUser',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new user in the system or updates an existing one.'
GO

IF OBJECT_ID('[dbo].[Users_CreateOrUpdateUser]') IS NOT NULL
	DROP PROCEDURE [dbo].[Users_CreateOrUpdateUser]
GO

SET ANSI_NULLS ON
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Users_CreateOrUpdateUser]
(
	@User_Name USERNAME,
	@Display_Name VARCHAR(100),
	@Email_Address VARCHAR(250) = NULL,
	@Groups_Xml XML,
	@ChangePasswordOnLogin_Indicator YNINDICATOR
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

	IF EXISTS(SELECT *
				FROM [Groups]
			   WHERE [Group_Name] = @User_Name)
		RAISERROR('Cannot create user %s; a group of the same name exists', 16, 1)

	IF EXISTS(SELECT *
				FROM [Users]
	            WITH (UPDLOCK)
			   WHERE [User_Name] = @User_Name)
	BEGIN

		UPDATE [Users]
		   SET [Display_Name] = @Display_Name,
			   [Email_Address] = @Email_Address,
			   [ChangePasswordOnLogin_Indicator] = @ChangePasswordOnLogin_Indicator
		 WHERE [User_Name] = @User_Name

		-- EXEC [Events_RaiseEvent] 'USRMDU', NULL, @User_Name

	END
	ELSE
	BEGIN

		INSERT INTO [Users]
		(
			[User_Name],
			[Display_Name],
			[Email_Address],
			[ChangePasswordOnLogin_Indicator]
		)
		VALUES
		(
			@User_Name,
			@Display_Name,
			@Email_Address,
			@ChangePasswordOnLogin_Indicator
		)

		-- EXEC [Events_RaiseEvent] 'USRNWU', NULL, @User_Name

	END

	DELETE
	  FROM [UserGroups]
	 WHERE [User_Name] = @User_Name

	INSERT INTO [UserGroups]
	(
		[User_Name],
		[Group_Name]
	)
	SELECT @User_Name
		  ,GX.[value]('.', 'VARCHAR(50)') [Group_Name]
	  FROM @Groups_Xml.[nodes]('/groups/group') AS GroupsXml(GX)

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [Users_CreateOrUpdateUser] TO [OtterUser_Role]
GO
