EXEC [dbo].[__AddStoredProcInfo]
    /* StoredProc_Name         */ 'Variables_GetVariablesAccessibleFromScope',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'DataTable',
    /* DataTableNames_Csv      */ 'Variables',
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Gets the variables that are defined in any of the specified scopes.'
GO


IF OBJECT_ID('[Variables_GetVariablesAccessibleFromScope]') IS NOT NULL DROP PROCEDURE [Variables_GetVariablesAccessibleFromScope]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [Variables_GetVariablesAccessibleFromScope]
(
    @Server_Id INT = NULL,
    @Environment_Id INT = NULL,
    @ServerRole_Id INT = NULL,
    @Job_Id INT = NULL,
    @Variable_Name VARCHAR(50) = NULL,
    @ExpandRolesAndEnvironments_Indicator YNINDICATOR = NULL,
    @IncludeSystemVariables_Indicator YNINDICATOR = NULL
)
AS
BEGIN
    SET NOCOUNT ON

    SELECT *
      FROM [Variables]
     WHERE ([Variable_Name] = @Variable_Name OR @Variable_Name IS NULL)
       AND ([Server_Id] = @Server_Id
            OR [Environment_Id] = @Environment_Id
            OR (@ExpandRolesAndEnvironments_Indicator = 'Y'
                AND [Environment_Id] IN (SELECT [Environment_Id] FROM dbo.[Environments_GetAncestorEnvironments](@Environment_Id)))
            OR [ServerRole_Id] = @ServerRole_Id
            OR (@ExpandRolesAndEnvironments_Indicator = 'Y'
                AND [ServerRole_Id] IN (SELECT [ServerRole_Id] FROM [ServerRoleServers] WHERE [Server_Id] = @Server_Id))
            OR [Job_Id] = @Job_Id
            OR (@IncludeSystemVariables_Indicator = 'Y' AND [Server_Id] IS NULL AND [Environment_Id] IS NULL AND [ServerRole_Id] IS NULL AND [Job_Id] IS NULL))

END
GO

GRANT EXECUTE ON [Variables_GetVariablesAccessibleFromScope] TO [OtterUser_Role]